@@ -1,5 +1,5 @@
 # util/langhelpers.py
-# Copyright (C) 2005-2016 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -9,23 +9,24 @@
 modules, classes, hierarchies, attributes, functions, and methods.
 
 """
-import itertools
+from functools import update_wrapper
+import hashlib
 import inspect
+import itertools
 import operator
 import re
 import sys
 import types
 import warnings
-from functools import update_wrapper
-from .. import exc
-import hashlib
-from . import compat
+
 from . import _collections
+from . import compat
+from .. import exc
 
 
 def md5_hex(x):
     if compat.py3k:
-        x = x.encode('utf-8')
+        x = x.encode("utf-8")
     m = hashlib.md5()
     m.update(x)
     return m.hexdigest()
@@ -49,6 +50,11 @@ class safe_reraise(object):
 
     """
 
+    __slots__ = ("warn_only", "_exc_info")
+
+    def __init__(self, warn_only=False):
+        self.warn_only = warn_only
+
     def __enter__(self):
         self._exc_info = sys.exc_info()
 
@@ -56,8 +62,9 @@ class safe_reraise(object):
         # see #2703 for notes
         if type_ is None:
             exc_type, exc_value, exc_tb = self._exc_info
-            self._exc_info = None   # remove potential circular references
-            compat.reraise(exc_type, exc_value, exc_tb)
+            self._exc_info = None  # remove potential circular references
+            if not self.warn_only:
+                compat.reraise(exc_type, exc_value, exc_tb)
         else:
             if not compat.py3k and self._exc_info and self._exc_info[1]:
                 # emulate Py3K's behavior of telling us when an exception
@@ -65,8 +72,9 @@ class safe_reraise(object):
                 warn(
                     "An exception has occurred during handling of a "
                     "previous exception.  The previous exception "
-                    "is:\n %s %s\n" % (self._exc_info[0], self._exc_info[1]))
-            self._exc_info = None   # remove potential circular references
+                    "is:\n %s %s\n" % (self._exc_info[0], self._exc_info[1])
+                )
+            self._exc_info = None  # remove potential circular references
             compat.reraise(type_, value, traceback)
 
 
@@ -78,7 +86,7 @@ def decode_slice(slc):
     """
     ret = []
     for x in slc.start, slc.stop, slc.step:
-        if hasattr(x, '__index__'):
+        if hasattr(x, "__index__"):
             x = x.__index__()
         ret.append(x)
     return tuple(ret)
@@ -87,9 +95,10 @@ def decode_slice(slc):
 def _unique_symbols(used, *bases):
     used = set(used)
     for base in bases:
-        pool = itertools.chain((base,),
-                               compat.itertools_imap(lambda i: base + str(i),
-                                                     range(1000)))
+        pool = itertools.chain(
+            (base,),
+            compat.itertools_imap(lambda i: base + str(i), range(1000)),
+        )
         for sym in pool:
             if sym not in used:
                 used.add(sym)
@@ -116,21 +125,25 @@ def decorator(target):
             raise Exception("not a decoratable function")
         spec = compat.inspect_getfullargspec(fn)
         names = tuple(spec[0]) + spec[1:3] + (fn.__name__,)
-        targ_name, fn_name = _unique_symbols(names, 'target', 'fn')
+        targ_name, fn_name = _unique_symbols(names, "target", "fn")
 
         metadata = dict(target=targ_name, fn=fn_name)
         metadata.update(format_argspec_plus(spec, grouped=False))
-        metadata['name'] = fn.__name__
-        code = """\
+        metadata["name"] = fn.__name__
+        code = (
+            """\
 def %(name)s(%(args)s):
     return %(target)s(%(fn)s, %(apply_kw)s)
-""" % metadata
-        decorated = _exec_code_in_env(code,
-                                      {targ_name: target, fn_name: fn},
-                                      fn.__name__)
-        decorated.__defaults__ = getattr(fn, 'im_func', fn).__defaults__
+"""
+            % metadata
+        )
+        decorated = _exec_code_in_env(
+            code, {targ_name: target, fn_name: fn}, fn.__name__
+        )
+        decorated.__defaults__ = getattr(fn, "im_func", fn).__defaults__
         decorated.__wrapped__ = fn
         return update_wrapper(decorated, fn)
+
     return update_wrapper(decorate, target)
 
 
@@ -149,31 +162,38 @@ def public_factory(target, location):
     if isinstance(target, type):
         fn = target.__init__
         callable_ = target
-        doc = "Construct a new :class:`.%s` object. \n\n"\
-            "This constructor is mirrored as a public API function; "\
-            "see :func:`~%s` "\
-            "for a full usage and argument description." % (
-                target.__name__, location, )
+        doc = (
+            "Construct a new :class:`.%s` object. \n\n"
+            "This constructor is mirrored as a public API function; "
+            "see :func:`~%s` "
+            "for a full usage and argument description."
+            % (target.__name__, location)
+        )
     else:
         fn = callable_ = target
-        doc = "This function is mirrored; see :func:`~%s` "\
+        doc = (
+            "This function is mirrored; see :func:`~%s` "
             "for a description of arguments." % location
+        )
 
     location_name = location.split(".")[-1]
     spec = compat.inspect_getfullargspec(fn)
     del spec[0][0]
     metadata = format_argspec_plus(spec, grouped=False)
-    metadata['name'] = location_name
-    code = """\
+    metadata["name"] = location_name
+    code = (
+        """\
 def %(name)s(%(args)s):
     return cls(%(apply_kw)s)
-""" % metadata
-    env = {'cls': callable_, 'symbol': symbol}
+"""
+        % metadata
+    )
+    env = {"cls": callable_, "symbol": symbol}
     exec(code, env)
     decorated = env[location_name]
     decorated.__doc__ = fn.__doc__
     decorated.__module__ = "sqlalchemy" + location.rsplit(".", 1)[0]
-    if compat.py2k or hasattr(fn, '__func__'):
+    if compat.py2k or hasattr(fn, "__func__"):
         fn.__func__.__doc__ = doc
     else:
         fn.__doc__ = doc
@@ -181,12 +201,14 @@ def %(name)s(%(args)s):
 
 
 class PluginLoader(object):
-
     def __init__(self, group, auto_fn=None):
         self.group = group
         self.impls = {}
         self.auto_fn = auto_fn
 
+    def clear(self):
+        self.impls.clear()
+
     def load(self, name):
         if name in self.impls:
             return self.impls[name]()
@@ -202,14 +224,13 @@ class PluginLoader(object):
         except ImportError:
             pass
         else:
-            for impl in pkg_resources.iter_entry_points(
-                    self.group, name):
+            for impl in pkg_resources.iter_entry_points(self.group, name):
                 self.impls[name] = impl.load
                 return impl.load()
 
         raise exc.NoSuchModuleError(
-            "Can't load plugin: %s:%s" %
-            (self.group, name))
+            "Can't load plugin: %s:%s" % (self.group, name)
+        )
 
     def register(self, name, modulepath, objname):
         def load():
@@ -217,11 +238,12 @@ class PluginLoader(object):
             for token in modulepath.split(".")[1:]:
                 mod = getattr(mod, token)
             return getattr(mod, objname)
+
         self.impls[name] = load
 
 
 def get_cls_kwargs(cls, _set=None):
-    """Return the full set of inherited kwargs for the given `cls`.
+    r"""Return the full set of inherited kwargs for the given `cls`.
 
     Probes a class's __init__ method, collecting all named arguments.  If the
     __init__ defines a \**kwargs catch-all, then the constructor is presumed
@@ -236,10 +258,13 @@ def get_cls_kwargs(cls, _set=None):
     if toplevel:
         _set = set()
 
-    ctr = cls.__dict__.get('__init__', False)
+    ctr = cls.__dict__.get("__init__", False)
 
-    has_init = ctr and isinstance(ctr, types.FunctionType) and \
-        isinstance(ctr.__code__, types.CodeType)
+    has_init = (
+        ctr
+        and isinstance(ctr, types.FunctionType)
+        and isinstance(ctr.__code__, types.CodeType)
+    )
 
     if has_init:
         names, has_kw = inspect_func_args(ctr)
@@ -253,7 +278,7 @@ def get_cls_kwargs(cls, _set=None):
             if get_cls_kwargs(c, _set) is None:
                 break
 
-    _set.discard('self')
+    _set.discard("self")
     return _set
 
 
@@ -269,9 +294,11 @@ try:
         has_kw = bool(co.co_flags & CO_VARKEYWORDS)
         return args, has_kw
 
+
 except ImportError:
+
     def inspect_func_args(fn):
-        names, _, has_kw, _ = inspect.getargspec(fn)
+        names, _, has_kw, _ = compat.inspect_getargspec(fn)
         return names, bool(has_kw)
 
 
@@ -300,23 +327,26 @@ def get_callable_argspec(fn, no_self=False, _is_init=False):
     elif inspect.isfunction(fn):
         if _is_init and no_self:
             spec = compat.inspect_getargspec(fn)
-            return compat.ArgSpec(spec.args[1:], spec.varargs,
-                                  spec.keywords, spec.defaults)
+            return compat.ArgSpec(
+                spec.args[1:], spec.varargs, spec.keywords, spec.defaults
+            )
         else:
             return compat.inspect_getargspec(fn)
     elif inspect.ismethod(fn):
         if no_self and (_is_init or fn.__self__):
             spec = compat.inspect_getargspec(fn.__func__)
-            return compat.ArgSpec(spec.args[1:], spec.varargs,
-                                  spec.keywords, spec.defaults)
+            return compat.ArgSpec(
+                spec.args[1:], spec.varargs, spec.keywords, spec.defaults
+            )
         else:
             return compat.inspect_getargspec(fn.__func__)
     elif inspect.isclass(fn):
         return get_callable_argspec(
-            fn.__init__, no_self=no_self, _is_init=True)
-    elif hasattr(fn, '__func__'):
+            fn.__init__, no_self=no_self, _is_init=True
+        )
+    elif hasattr(fn, "__func__"):
         return compat.inspect_getargspec(fn.__func__)
-    elif hasattr(fn, '__call__'):
+    elif hasattr(fn, "__call__"):
         if inspect.ismethod(fn.__call__):
             return get_callable_argspec(fn.__call__, no_self=no_self)
         else:
@@ -362,17 +392,18 @@ def format_argspec_plus(fn, grouped=True):
     else:
         # we accept an existing argspec...
         spec = fn
-    args = inspect.formatargspec(*spec)
+    args = compat.inspect_formatargspec(*spec)
     if spec[0]:
         self_arg = spec[0][0]
     elif spec[1]:
-        self_arg = '%s[0]' % spec[1]
+        self_arg = "%s[0]" % spec[1]
     else:
         self_arg = None
 
     if compat.py3k:
-        apply_pos = inspect.formatargspec(spec[0], spec[1],
-                                          spec[2], None, spec[4])
+        apply_pos = compat.inspect_formatargspec(
+            spec[0], spec[1], spec[2], None, spec[4]
+        )
         num_defaults = 0
         if spec[3]:
             num_defaults += len(spec[3])
@@ -380,26 +411,38 @@ def format_argspec_plus(fn, grouped=True):
             num_defaults += len(spec[4])
         name_args = spec[0] + spec[4]
     else:
-        apply_pos = inspect.formatargspec(spec[0], spec[1], spec[2])
+        apply_pos = compat.inspect_formatargspec(spec[0], spec[1], spec[2])
         num_defaults = 0
         if spec[3]:
             num_defaults += len(spec[3])
         name_args = spec[0]
 
     if num_defaults:
-        defaulted_vals = name_args[0 - num_defaults:]
+        defaulted_vals = name_args[0 - num_defaults :]
     else:
         defaulted_vals = ()
 
-    apply_kw = inspect.formatargspec(name_args, spec[1], spec[2],
-                                     defaulted_vals,
-                                     formatvalue=lambda x: '=' + x)
+    apply_kw = compat.inspect_formatargspec(
+        name_args,
+        spec[1],
+        spec[2],
+        defaulted_vals,
+        formatvalue=lambda x: "=" + x,
+    )
     if grouped:
-        return dict(args=args, self_arg=self_arg,
-                    apply_pos=apply_pos, apply_kw=apply_kw)
+        return dict(
+            args=args,
+            self_arg=self_arg,
+            apply_pos=apply_pos,
+            apply_kw=apply_kw,
+        )
     else:
-        return dict(args=args[1:-1], self_arg=self_arg,
-                    apply_pos=apply_pos[1:-1], apply_kw=apply_kw[1:-1])
+        return dict(
+            args=args[1:-1],
+            self_arg=self_arg,
+            apply_pos=apply_pos[1:-1],
+            apply_kw=apply_kw[1:-1],
+        )
 
 
 def format_argspec_init(method, grouped=True):
@@ -413,14 +456,17 @@ def format_argspec_init(method, grouped=True):
 
     """
     if method is object.__init__:
-        args = grouped and '(self)' or 'self'
+        args = grouped and "(self)" or "self"
     else:
         try:
             return format_argspec_plus(method, grouped=grouped)
         except TypeError:
-            args = (grouped and '(self, *args, **kwargs)'
-                    or 'self, *args, **kwargs')
-    return dict(self_arg='self', args=args, apply_pos=args, apply_kw=args)
+            args = (
+                grouped
+                and "(self, *args, **kwargs)"
+                or "self, *args, **kwargs"
+            )
+    return dict(self_arg="self", args=args, apply_pos=args, apply_kw=args)
 
 
 def getargspec_init(method):
@@ -436,9 +482,9 @@ def getargspec_init(method):
         return compat.inspect_getargspec(method)
     except TypeError:
         if method is object.__init__:
-            return (['self'], None, None, None)
+            return (["self"], None, None, None)
         else:
-            return (['self'], 'args', 'kwargs', None)
+            return (["self"], "args", "kwargs", None)
 
 
 def unbound_method_to_callable(func_or_cls):
@@ -470,8 +516,9 @@ def generic_repr(obj, additional_kw=(), to_inspect=None, omit_kwarg=()):
     vargs = None
     for i, insp in enumerate(to_inspect):
         try:
-            (_args, _vargs, vkw, defaults) = \
-                compat.inspect_getargspec(insp.__init__)
+            (_args, _vargs, vkw, defaults) = compat.inspect_getargspec(
+                insp.__init__
+            )
         except TypeError:
             continue
         else:
@@ -484,16 +531,17 @@ def generic_repr(obj, additional_kw=(), to_inspect=None, omit_kwarg=()):
                 else:
                     pos_args.extend(_args[1:])
             else:
-                kw_args.update([
-                    (arg, missing) for arg in _args[1:-default_len]
-                ])
+                kw_args.update(
+                    [(arg, missing) for arg in _args[1:-default_len]]
+                )
 
             if default_len:
-                kw_args.update([
-                    (arg, default)
-                    for arg, default
-                    in zip(_args[-default_len:], defaults)
-                ])
+                kw_args.update(
+                    [
+                        (arg, default)
+                        for arg, default in zip(_args[-default_len:], defaults)
+                    ]
+                )
     output = []
 
     output.extend(repr(getattr(obj, arg, None)) for arg in pos_args)
@@ -507,7 +555,7 @@ def generic_repr(obj, additional_kw=(), to_inspect=None, omit_kwarg=()):
         try:
             val = getattr(obj, arg, missing)
             if val is not missing and val != defval:
-                output.append('%s=%r' % (arg, val))
+                output.append("%s=%r" % (arg, val))
         except Exception:
             pass
 
@@ -516,7 +564,7 @@ def generic_repr(obj, additional_kw=(), to_inspect=None, omit_kwarg=()):
             try:
                 val = getattr(obj, arg, missing)
                 if val is not missing and val != defval:
-                    output.append('%s=%r' % (arg, val))
+                    output.append("%s=%r" % (arg, val))
             except Exception:
                 pass
 
@@ -529,20 +577,27 @@ class portable_instancemethod(object):
 
     """
 
-    __slots__ = 'target', 'name', '__weakref__'
+    __slots__ = "target", "name", "kwargs", "__weakref__"
 
     def __getstate__(self):
-        return {'target': self.target, 'name': self.name}
+        return {
+            "target": self.target,
+            "name": self.name,
+            "kwargs": self.kwargs,
+        }
 
     def __setstate__(self, state):
-        self.target = state['target']
-        self.name = state['name']
+        self.target = state["target"]
+        self.name = state["name"]
+        self.kwargs = state.get("kwargs", ())
 
-    def __init__(self, meth):
+    def __init__(self, meth, kwargs=()):
         self.target = meth.__self__
         self.name = meth.__name__
+        self.kwargs = kwargs
 
     def __call__(self, *arg, **kw):
+        kw.update(self.kwargs)
         return getattr(self.target, self.name)(*arg, **kw)
 
 
@@ -563,15 +618,18 @@ def class_hierarchy(cls):
         if isinstance(cls, types.ClassType):
             return list()
 
-    hier = set([cls])
+    hier = {cls}
     process = list(cls.__mro__)
     while process:
         c = process.pop()
         if compat.py2k:
             if isinstance(c, types.ClassType):
                 continue
-            bases = (_ for _ in c.__bases__
-                     if _ not in hier and not isinstance(_, types.ClassType))
+            bases = (
+                _
+                for _ in c.__bases__
+                if _ not in hier and not isinstance(_, types.ClassType)
+            )
         else:
             bases = (_ for _ in c.__bases__ if _ not in hier)
 
@@ -580,11 +638,12 @@ def class_hierarchy(cls):
             hier.add(b)
 
         if compat.py3k:
-            if c.__module__ == 'builtins' or not hasattr(c, '__subclasses__'):
+            if c.__module__ == "builtins" or not hasattr(c, "__subclasses__"):
                 continue
         else:
-            if c.__module__ == '__builtin__' or not hasattr(
-                    c, '__subclasses__'):
+            if c.__module__ == "__builtin__" or not hasattr(
+                c, "__subclasses__"
+            ):
                 continue
 
         for s in [_ for _ in c.__subclasses__() if _ not in hier]:
@@ -609,38 +668,59 @@ def iterate_attributes(cls):
                 break
 
 
-def monkeypatch_proxied_specials(into_cls, from_cls, skip=None, only=None,
-                                 name='self.proxy', from_instance=None):
+def monkeypatch_proxied_specials(
+    into_cls,
+    from_cls,
+    skip=None,
+    only=None,
+    name="self.proxy",
+    from_instance=None,
+):
     """Automates delegation of __specials__ for a proxying type."""
 
     if only:
         dunders = only
     else:
         if skip is None:
-            skip = ('__slots__', '__del__', '__getattribute__',
-                    '__metaclass__', '__getstate__', '__setstate__')
-        dunders = [m for m in dir(from_cls)
-                   if (m.startswith('__') and m.endswith('__') and
-                       not hasattr(into_cls, m) and m not in skip)]
+            skip = (
+                "__slots__",
+                "__del__",
+                "__getattribute__",
+                "__metaclass__",
+                "__getstate__",
+                "__setstate__",
+            )
+        dunders = [
+            m
+            for m in dir(from_cls)
+            if (
+                m.startswith("__")
+                and m.endswith("__")
+                and not hasattr(into_cls, m)
+                and m not in skip
+            )
+        ]
 
     for method in dunders:
         try:
             fn = getattr(from_cls, method)
-            if not hasattr(fn, '__call__'):
+            if not hasattr(fn, "__call__"):
                 continue
-            fn = getattr(fn, 'im_func', fn)
+            fn = getattr(fn, "im_func", fn)
         except AttributeError:
             continue
         try:
             spec = compat.inspect_getargspec(fn)
-            fn_args = inspect.formatargspec(spec[0])
-            d_args = inspect.formatargspec(spec[0][1:])
+            fn_args = compat.inspect_formatargspec(spec[0])
+            d_args = compat.inspect_formatargspec(spec[0][1:])
         except TypeError:
-            fn_args = '(self, *args, **kw)'
-            d_args = '(*args, **kw)'
+            fn_args = "(self, *args, **kw)"
+            d_args = "(*args, **kw)"
 
-        py = ("def %(method)s%(fn_args)s: "
-              "return %(name)s.%(method)s%(d_args)s" % locals())
+        py = (
+            "def %(method)s%(fn_args)s: "
+            "return %(name)s.%(method)s%(d_args)s" % locals()
+        )
 
         env = from_instance is not None and {name: from_instance} or {}
         compat.exec_(py, env)
@@ -654,8 +734,9 @@ def monkeypatch_proxied_specials(into_cls, from_cls, skip=None, only=None,
 def methods_equivalent(meth1, meth2):
     """Return True if the two methods are the same implementation."""
 
-    return getattr(meth1, '__func__', meth1) is getattr(
-        meth2, '__func__', meth2)
+    return getattr(meth1, "__func__", meth1) is getattr(
+        meth2, "__func__", meth2
+    )
 
 
 def as_interface(obj, cls=None, methods=None, required=None):
@@ -692,12 +773,12 @@ def as_interface(obj, cls=None, methods=None, required=None):
 
     """
     if not cls and not methods:
-        raise TypeError('a class or collection of method names are required')
+        raise TypeError("a class or collection of method names are required")
 
     if isinstance(cls, type) and isinstance(obj, cls):
         return obj
 
-    interface = set(methods or [m for m in dir(cls) if not m.startswith('_')])
+    interface = set(methods or [m for m in dir(cls) if not m.startswith("_")])
     implemented = set(dir(obj))
 
     complies = operator.ge
@@ -714,15 +795,17 @@ def as_interface(obj, cls=None, methods=None, required=None):
 
     # No dict duck typing here.
     if not isinstance(obj, dict):
-        qualifier = complies is operator.gt and 'any of' or 'all of'
-        raise TypeError("%r does not implement %s: %s" % (
-            obj, qualifier, ', '.join(interface)))
+        qualifier = complies is operator.gt and "any of" or "all of"
+        raise TypeError(
+            "%r does not implement %s: %s"
+            % (obj, qualifier, ", ".join(interface))
+        )
 
     class AnonymousInterface(object):
         """A callable-holding shell."""
 
     if cls:
-        AnonymousInterface.__name__ = 'Anonymous' + cls.__name__
+        AnonymousInterface.__name__ = "Anonymous" + cls.__name__
     found = set()
 
     for method, impl in dictlike_iteritems(obj):
@@ -736,8 +819,10 @@ def as_interface(obj, cls=None, methods=None, required=None):
     if complies(found, required):
         return AnonymousInterface
 
-    raise TypeError("dictionary does not contain required keys %s" %
-                    ', '.join(required - found))
+    raise TypeError(
+        "dictionary does not contain required keys %s"
+        % ", ".join(required - found)
+    )
 
 
 class memoized_property(object):
@@ -773,11 +858,15 @@ def memoized_instancemethod(fn):
 
     def oneshot(self, *args, **kw):
         result = fn(self, *args, **kw)
-        memo = lambda *a, **kw: result
+
+        def memo(*a, **kw):
+            return result
+
         memo.__name__ = fn.__name__
         memo.__doc__ = fn.__doc__
         self.__dict__[fn.__name__] = memo
         return result
+
     return update_wrapper(oneshot, fn)
 
 
@@ -818,38 +907,44 @@ class MemoizedSlots(object):
         raise AttributeError(key)
 
     def __getattr__(self, key):
-        if key.startswith('_memoized'):
+        if key.startswith("_memoized"):
             raise AttributeError(key)
-        elif hasattr(self, '_memoized_attr_%s' % key):
-            value = getattr(self, '_memoized_attr_%s' % key)()
+        elif hasattr(self, "_memoized_attr_%s" % key):
+            value = getattr(self, "_memoized_attr_%s" % key)()
             setattr(self, key, value)
             return value
-        elif hasattr(self, '_memoized_method_%s' % key):
-            fn = getattr(self, '_memoized_method_%s' % key)
+        elif hasattr(self, "_memoized_method_%s" % key):
+            fn = getattr(self, "_memoized_method_%s" % key)
 
             def oneshot(*args, **kw):
                 result = fn(*args, **kw)
-                memo = lambda *a, **kw: result
+
+                def memo(*a, **kw):
+                    return result
+
                 memo.__name__ = fn.__name__
                 memo.__doc__ = fn.__doc__
                 setattr(self, key, memo)
                 return result
+
             oneshot.__doc__ = fn.__doc__
             return oneshot
         else:
             return self._fallback_getattr(key)
 
 
-def dependency_for(modulename):
+def dependency_for(modulename, add_to_all=False):
     def decorate(obj):
-        # TODO: would be nice to improve on this import silliness,
-        # unfortunately importlib doesn't work that great either
         tokens = modulename.split(".")
         mod = compat.import_(
-            ".".join(tokens[0:-1]), globals(), locals(), tokens[-1])
+            ".".join(tokens[0:-1]), globals(), locals(), [tokens[-1]]
+        )
         mod = getattr(mod, tokens[-1])
         setattr(mod, obj.__name__, obj)
+        if add_to_all and hasattr(mod, "__all__"):
+            mod.__all__.append(obj.__name__)
         return obj
+
     return decorate
 
 
@@ -876,10 +971,7 @@ class dependencies(object):
         for dep in deps:
             tokens = dep.split(".")
             self.import_deps.append(
-                dependencies._importlater(
-                    ".".join(tokens[0:-1]),
-                    tokens[-1]
-                )
+                dependencies._importlater(".".join(tokens[0:-1]), tokens[-1])
             )
 
     def __call__(self, fn):
@@ -887,7 +979,7 @@ class dependencies(object):
         spec = compat.inspect_getfullargspec(fn)
 
         spec_zero = list(spec[0])
-        hasself = spec_zero[0] in ('self', 'cls')
+        hasself = spec_zero[0] in ("self", "cls")
 
         for i in range(len(import_deps)):
             spec[0][i + (1 if hasself else 0)] = "import_deps[%r]" % i
@@ -900,13 +992,13 @@ class dependencies(object):
 
         outer_spec = format_argspec_plus(spec, grouped=False)
 
-        code = 'lambda %(args)s: fn(%(apply_kw)s)' % {
-            "args": outer_spec['args'],
-            "apply_kw": inner_spec['apply_kw']
+        code = "lambda %(args)s: fn(%(apply_kw)s)" % {
+            "args": outer_spec["args"],
+            "apply_kw": inner_spec["apply_kw"],
         }
 
         decorated = eval(code, locals())
-        decorated.__defaults__ = getattr(fn, 'im_func', fn).__defaults__
+        decorated.__defaults__ = getattr(fn, "im_func", fn).__defaults__
         return update_wrapper(decorated, fn)
 
     @classmethod
@@ -946,26 +1038,27 @@ class dependencies(object):
                 raise ImportError(
                     "importlater.resolve_all() hasn't "
                     "been called (this is %s %s)"
-                    % (self._il_path, self._il_addtl))
+                    % (self._il_path, self._il_addtl)
+                )
 
             return getattr(self._initial_import, self._il_addtl)
 
         def _resolve(self):
             dependencies._unresolved.discard(self)
             self._initial_import = compat.import_(
-                self._il_path, globals(), locals(),
-                [self._il_addtl])
+                self._il_path, globals(), locals(), [self._il_addtl]
+            )
 
         def __getattr__(self, key):
-            if key == 'module':
-                raise ImportError("Could not resolve module %s"
-                                  % self._full_path)
+            if key == "module":
+                raise ImportError(
+                    "Could not resolve module %s" % self._full_path
+                )
             try:
                 attr = getattr(self.module, key)
             except AttributeError:
                 raise AttributeError(
-                    "Module %s has no attribute '%s'" %
-                    (self._full_path, key)
+                    "Module %s has no attribute '%s'" % (self._full_path, key)
                 )
             self.__dict__[key] = attr
             return attr
@@ -975,9 +1068,9 @@ class dependencies(object):
 def asbool(obj):
     if isinstance(obj, compat.string_types):
         obj = obj.strip().lower()
-        if obj in ['true', 'yes', 'on', 'y', 't', '1']:
+        if obj in ["true", "yes", "on", "y", "t", "1"]:
             return True
-        elif obj in ['false', 'no', 'off', 'n', 'f', '0']:
+        elif obj in ["false", "no", "off", "n", "f", "0"]:
             return False
         else:
             raise ValueError("String is not true/false: %r" % obj)
@@ -989,11 +1082,13 @@ def bool_or_str(*text):
     boolean, or one of a set of "alternate" string values.
 
     """
+
     def bool_or_value(obj):
         if obj in text:
             return obj
         else:
             return asbool(obj)
+
     return bool_or_value
 
 
@@ -1006,7 +1101,7 @@ def asint(value):
 
 
 def coerce_kw_type(kw, key, type_, flexi_bool=True):
-    """If 'key' is present in dict 'kw', coerce its value to type 'type\_' if
+    r"""If 'key' is present in dict 'kw', coerce its value to type 'type\_' if
     necessary.  If 'flexi_bool' is True, the string '0' is considered false
     when coercing to boolean.
     """
@@ -1027,8 +1122,8 @@ def constructor_copy(obj, cls, *args, **kw):
 
     names = get_cls_kwargs(cls)
     kw.update(
-        (k, obj.__dict__[k]) for k in names.difference(kw)
-        if k in obj.__dict__)
+        (k, obj.__dict__[k]) for k in names.difference(kw) if k in obj.__dict__
+    )
     return cls(*args, **kw)
 
 
@@ -1055,10 +1150,11 @@ def duck_type_collection(specimen, default=None):
     property is present, return that preferentially.
     """
 
-    if hasattr(specimen, '__emulates__'):
+    if hasattr(specimen, "__emulates__"):
         # canonicalize set vs sets.Set to a standard: the builtin set
-        if (specimen.__emulates__ is not None and
-                issubclass(specimen.__emulates__, set)):
+        if specimen.__emulates__ is not None and issubclass(
+            specimen.__emulates__, set
+        ):
             return set
         else:
             return specimen.__emulates__
@@ -1071,11 +1167,11 @@ def duck_type_collection(specimen, default=None):
     elif isa(specimen, dict):
         return dict
 
-    if hasattr(specimen, 'append'):
+    if hasattr(specimen, "append"):
         return list
-    elif hasattr(specimen, 'add'):
+    elif hasattr(specimen, "add"):
         return set
-    elif hasattr(specimen, 'set'):
+    elif hasattr(specimen, "set"):
         return dict
     else:
         return default
@@ -1087,41 +1183,43 @@ def assert_arg_type(arg, argtype, name):
     else:
         if isinstance(argtype, tuple):
             raise exc.ArgumentError(
-                "Argument '%s' is expected to be one of type %s, got '%s'" %
-                (name, ' or '.join("'%s'" % a for a in argtype), type(arg)))
+                "Argument '%s' is expected to be one of type %s, got '%s'"
+                % (name, " or ".join("'%s'" % a for a in argtype), type(arg))
+            )
         else:
             raise exc.ArgumentError(
-                "Argument '%s' is expected to be of type '%s', got '%s'" %
-                (name, argtype, type(arg)))
+                "Argument '%s' is expected to be of type '%s', got '%s'"
+                % (name, argtype, type(arg))
+            )
 
 
 def dictlike_iteritems(dictlike):
     """Return a (key, value) iterator for almost any dict-like object."""
 
     if compat.py3k:
-        if hasattr(dictlike, 'items'):
+        if hasattr(dictlike, "items"):
             return list(dictlike.items())
     else:
-        if hasattr(dictlike, 'iteritems'):
+        if hasattr(dictlike, "iteritems"):
             return dictlike.iteritems()
-        elif hasattr(dictlike, 'items'):
+        elif hasattr(dictlike, "items"):
             return iter(dictlike.items())
 
-    getter = getattr(dictlike, '__getitem__', getattr(dictlike, 'get', None))
+    getter = getattr(dictlike, "__getitem__", getattr(dictlike, "get", None))
     if getter is None:
-        raise TypeError(
-            "Object '%r' is not dict-like" % dictlike)
+        raise TypeError("Object '%r' is not dict-like" % dictlike)
+
+    if hasattr(dictlike, "iterkeys"):
 
-    if hasattr(dictlike, 'iterkeys'):
         def iterator():
             for key in dictlike.iterkeys():
                 yield key, getter(key)
+
         return iterator()
-    elif hasattr(dictlike, 'keys'):
+    elif hasattr(dictlike, "keys"):
         return iter((key, getter(key)) for key in dictlike.keys())
     else:
-        raise TypeError(
-            "Object '%r' is not dict-like" % dictlike)
+        raise TypeError("Object '%r' is not dict-like" % dictlike)
 
 
 class classproperty(property):
@@ -1190,7 +1288,8 @@ class _symbol(int):
     def __repr__(self):
         return "symbol(%r)" % self.name
 
-_symbol.__name__ = 'symbol'
+
+_symbol.__name__ = "symbol"
 
 
 class symbol(object):
@@ -1214,6 +1313,7 @@ class symbol(object):
     ``doc`` here.
 
     """
+
     symbols = {}
     _lock = compat.threading.Lock()
 
@@ -1275,9 +1375,11 @@ class _hash_limit_string(compat.text_type):
 
 
     """
+
     def __new__(cls, value, num, args):
-        interpolated = (value % args) + \
-            (" (this warning may be suppressed after %d occurrences)" % num)
+        interpolated = (value % args) + (
+            " (this warning may be suppressed after %d occurrences)" % num
+        )
         self = super(_hash_limit_string, cls).__new__(cls, interpolated)
         self._hash = hash("%s_%d" % (value, hash(interpolated) % num))
         return self
@@ -1323,8 +1425,8 @@ def only_once(fn):
     return go
 
 
-_SQLA_RE = re.compile(r'sqlalchemy/([a-z_]+/){0,2}[a-z_]+\.py')
-_UNITTEST_RE = re.compile(r'unit(?:2|test2?/)')
+_SQLA_RE = re.compile(r"sqlalchemy/([a-z_]+/){0,2}[a-z_]+\.py")
+_UNITTEST_RE = re.compile(r"unit(?:2|test2?/)")
 
 
 def chop_traceback(tb, exclude_prefix=_UNITTEST_RE, exclude_suffix=_SQLA_RE):
@@ -1346,25 +1448,25 @@ def chop_traceback(tb, exclude_prefix=_UNITTEST_RE, exclude_suffix=_SQLA_RE):
         start += 1
     while start <= end and exclude_suffix.search(tb[end]):
         end -= 1
-    return tb[start:end + 1]
+    return tb[start : end + 1]
+
 
 NoneType = type(None)
 
 
 def attrsetter(attrname):
-    code = \
-        "def set(obj, value):"\
-        "    obj.%s = value" % attrname
+    code = "def set(obj, value):" "    obj.%s = value" % attrname
     env = locals().copy()
     exec(code, env)
-    return env['set']
+    return env["set"]
 
 
 class EnsureKWArgType(type):
-    """Apply translation of functions to accept **kw arguments if they
+    r"""Apply translation of functions to accept \**kw arguments if they
     don't already.
 
     """
+
     def __init__(cls, clsname, bases, clsdict):
         fn_reg = cls.ensure_kwarg
         if fn_reg:
@@ -1379,9 +1481,9 @@ class EnsureKWArgType(type):
         super(EnsureKWArgType, cls).__init__(clsname, bases, clsdict)
 
     def _wrap_w_kw(self, fn):
-
         def wrap(*arg, **kw):
             return fn(*arg)
+
         return update_wrapper(wrap, fn)
 
 
@@ -1393,17 +1495,61 @@ def wrap_callable(wrapper, fn):
       object with __call__ method
 
     """
-    if hasattr(fn, '__name__'):
+    if hasattr(fn, "__name__"):
         return update_wrapper(wrapper, fn)
     else:
         _f = wrapper
         _f.__name__ = fn.__class__.__name__
-        if hasattr(fn, '__module__'):
+        if hasattr(fn, "__module__"):
             _f.__module__ = fn.__module__
 
-        if hasattr(fn.__call__, '__doc__') and fn.__call__.__doc__:
+        if hasattr(fn.__call__, "__doc__") and fn.__call__.__doc__:
             _f.__doc__ = fn.__call__.__doc__
         elif fn.__doc__:
             _f.__doc__ = fn.__doc__
 
         return _f
+
+
+def quoted_token_parser(value):
+    """Parse a dotted identifier with accomodation for quoted names.
+
+    Includes support for SQL-style double quotes as a literal character.
+
+    E.g.::
+
+        >>> quoted_token_parser("name")
+        ["name"]
+        >>> quoted_token_parser("schema.name")
+        ["schema", "name"]
+        >>> quoted_token_parser('"Schema"."Name"')
+        ['Schema', 'Name']
+        >>> quoted_token_parser('"Schema"."Name""Foo"')
+        ['Schema', 'Name""Foo']
+
+    """
+
+    if '"' not in value:
+        return value.split(".")
+
+    # 0 = outside of quotes
+    # 1 = inside of quotes
+    state = 0
+    result = [[]]
+    idx = 0
+    lv = len(value)
+    while idx < lv:
+        char = value[idx]
+        if char == '"':
+            if state == 1 and idx < lv - 1 and value[idx + 1] == '"':
+                result[-1].append('"')
+                idx += 1
+            else:
+                state ^= 1
+        elif char == "." and state == 0:
+            result.append([])
+        else:
+            result[-1].append(char)
+        idx += 1
+
+    return ["".join(token) for token in result]
